/**
 * Copyright (c) 2000-2007 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.tools;

import java.io.File;

import com.liferay.portal.util.FileImpl;

/**
 * <a href="CSSBuilder.java.html"><b><i>View Source</i></b></a>
 * @version Developed (based) on the libraries coming with Liferay 5.1.0
 * @author Brian Wing Shun Chan, Artur Linhart
 * 
 */
public class CSSBuilder {

	public static void main(String[] args) {
		if (args.length == 2) {
			new CSSBuilder(args[0], args[1]);
		} else {
			throw new IllegalArgumentException();
		}
	}

	public CSSBuilder(String cssDir, String mergedFile) {
		try {
			System.out.println("Packing css files from root folder '" + cssDir + "'");
			System.out.println("                         into file '" + mergedFile + "'...");
			
            File mainCssFile = new File(cssDir + "/main.css");

            if (!mainCssFile.exists()) {
                    System.out.println(
                            "Do not pack " + cssDir +
                                    "/main.css because it does not exist.");

                    return;
            }

            FileImpl fileImpl = new FileImpl();

            String content = fileImpl.read(mainCssFile);

			content = replaceImports(cssDir, content);

			fileImpl.write(mergedFile, content);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Original method from Liferay
	 * @param cssDir root directory where the css have to be processed
	 * @param s css source which has to be processed
	 * @return modified css source
	 * @throws Exception if some error occurs
	 */
	public String replaceImports(String cssDir, String s) throws Exception {
		return replaceImports(cssDir, s, "");
	}
	
	/**
	 * Modified method used for the recursive conversion of the css-documents included in subfolders of the root folder
	 * @param cssDir root directory where the css have to be processed
	 * @param s css source which has to be processed
	 * @param aProcessedFileRelativePath
	 * @return converted css source code
	 * @throws Exception if some error occurs
	 */
	String replaceImports(String cssDir, String s,
			String aProcessedFileRelativePath) throws Exception {

		s = replaceUrlRelativePaths(s, aProcessedFileRelativePath);

		StringBuilder sm = new StringBuilder(s.length());
		int pos = 0;

		while (true) {
			int x = s.indexOf(IMPORT_BEGIN, pos);
			int y = s.indexOf(IMPORT_END, x + IMPORT_BEGIN.length());

			if ((x == -1) || (y == -1)) {
				sm.append(s.substring(pos, s.length()));

				break;
			} else {
				sm.append(s.substring(pos, x));

				// String importFile = s.substring(x + _BEGIN.length(), y);
				String importFile = resolveUrlStringFromIndex(s, x
						+ IMPORT_BEGIN.length(), "", false);
				String importFileRelativePath = extractFileUrlFolderPath(importFile);

				String importContent = _fileUtil.read(
                        cssDir + "/" + importFile);

				/* finally process all files, included in the content: */
				if (aProcessedFileRelativePath != null
						&& !"".equals(aProcessedFileRelativePath))
					importFileRelativePath = aProcessedFileRelativePath + "/"
							+ importFileRelativePath;
				importContent = replaceImports(cssDir, importContent,
						importFileRelativePath);

				sm.append(importContent);

				pos = y + IMPORT_END.length();
			}
		}

		return sm.toString();
	}

	/**
	 * Replaces the url relative paths (not containing the string "://" and not
	 * starting with "/" or "\" or having ":" on the second position) by the
	 * url, whcih includes in the beginning also the relative path of the
	 * currently processed file, given by the second parameter. As separator is
	 * used the character "/"
	 * 
	 * @param aContent
	 *            content which has to be searched
	 * @param aProcessedFileRelativePath
	 *            relative path of the currently processed content which has to
	 *            be appended to the before all relative path
	 * @return the processed content. In the case the relative path is empty
	 *         string or <code>null</code>, then the same pointer is returned
	 *         like put into the first parameter without any processing.
	 * @author Artur Linhart
	 */
	static String replaceUrlRelativePaths(String aContent,
			String aProcessedFileRelativePath) {

		/*
		 * replace the relative paths in the currently processed file - but only
		 * in the case if it is located in another than the default folder:
		 */
		if (aProcessedFileRelativePath == null
				|| "".equals(aProcessedFileRelativePath))
			return aContent;
		else {
			StringBuilder sm = new StringBuilder(aContent.length());
			int pos = 0;
			while (true) {
				int x = aContent.indexOf(URL_BEGIN_PREFIX, pos);
				if (x == -1) {
					sm.append(aContent.substring(pos, aContent.length()));

					break;
				} else {
					sm.append(aContent.substring(pos, x
							+ URL_BEGIN_PREFIX_LENGTH));
					pos = x + URL_BEGIN_PREFIX_LENGTH;
					while (pos < aContent.length()
							&& " ".equals(aContent.substring(pos, pos + 1))) {
						sm.append(" ");
						pos++;
					}
					if (pos < aContent.length()
							&& URL_BEGIN_CHAR.equals(aContent.substring(pos,
									pos + 1))) {
						// String importFile = s.substring(x + _BEGIN.length(),
						// y);
						String locUrlContent = resolveUrlStringFromIndex(
								aContent, pos, "", true);
						if (!"".equals(locUrlContent)) {
							int locTmpIndex = aContent.indexOf(locUrlContent,
									pos);
							/*
							 * append all characters from the begining to the
							 * start of the url. This is because there can be
							 * not only '(', but also '("' or '( "' etc.:
							 */
							sm.append(aContent.substring(pos, locTmpIndex));
							pos = locTmpIndex;
							/*
							 * Now there has been written everything till the
							 * start of the url - so if this is the relative
							 * url, then write before it the relative path of
							 * the currently processed file:
							 */
							if (!locUrlContent.contains("://")
									&& !locUrlContent.startsWith("/")
									&& !locUrlContent.startsWith("\\")
									&& !locUrlContent.startsWith(":", 1)) {
								sm.append(aProcessedFileRelativePath);
								sm.append("/");
							}
							/* and then write there the given url content */
							sm.append(locUrlContent);
							pos = pos + locUrlContent.length();
							/*
							 * and continue with the append of all other
							 * characters...
							 */
						}
					}
				}
			}
			return sm.toString();
		}
	}

	/**
	 * Extracts from the string url the folder path without the final filename.
	 * In the case the argument contains the folder path name closed by the
	 * folder separator "\" or "/", then the same string is returned. The method
	 * works also for the url containing parameters splitted from the url by the
	 * character "?".
	 * 
	 * @param aFileNameWithUrlPath
	 *            url from which the path has to be extracted
	 * @return url path of the file specified by the argument. In the case the
	 *         argument cotnains the argument finished by the folder separator,
	 *         then the same string is returned
	 * @author Artur Linhart
	 */
	static String extractFileUrlFolderPath(String aFileNameWithUrlPath) {
		if (aFileNameWithUrlPath == null)
			return null;
		int locLastPossibleFileSeparatorPosition = aFileNameWithUrlPath
				.indexOf("?"); /*
								 * for the case the path is the parametrized
								 * url, then search before the ? character only
								 */
		if (locLastPossibleFileSeparatorPosition < 0)
			locLastPossibleFileSeparatorPosition = aFileNameWithUrlPath
					.length();
		/* find the position of the last separator: */
		for (int locI = locLastPossibleFileSeparatorPosition - 1; locI >= 0; locI--) {
			if ("/".equals(aFileNameWithUrlPath.substring(locI, locI + 1))
					|| "\\".equals(aFileNameWithUrlPath.substring(locI,
							locI + 1))) {
				/* this is the last separator - extract the result */
				return aFileNameWithUrlPath.substring(0, locI);
			}
		}
		return "";
	}

	/**
	 * Resolves from the string containing the url with the possible starting
	 * and ending characters the concrete value of the url. It searches in the
	 * string given by the first argument from the position given by the second
	 * argument till the position of the string given by the third character. In
	 * the case the third character is empty string, then the method expects the
	 * url will be started by some combination of the begin characters given by
	 * {@link #BEGIN_CHARS}. It first ignores the leading spaces and then
	 * expects some from the character from {@link #BEGIN_CHARS} (for example
	 * '(' or '"') or some from the url prefixes, given by
	 * {@link #ALTERNATE_BEGIN_CHAR_PREFIX} (for example url). If this prefix is
	 * ignored or not this is given b the third parameter.
	 * 
	 * @param aStringContainingUrl
	 *            String containing the url which has to be searched
	 * @param aIndexOfFirstCheckedCharacter
	 *            first position in the strign given by the previous argument,
	 *            from which the url has to be searched
	 * @param aMasterEndCharacter
	 *            end character till which has to be searched on the current
	 *            resolution level
	 * @param aIgnoreBeginCharPrefix
	 *            gives the information, if the pregixes given by
	 *            {@link #ALTERNATE_BEGIN_CHAR_PREFIX} have to be considered or
	 *            ignored. It is usefull in the case the prefix has been already
	 *            separated (if they are on the position which will be not
	 *            checked during this method) or in the case the inside content
	 *            is checked already. It allows to resolve also such url
	 *            contents, specified for example by the url '<code>  url  (  "  url  "  )  </code>'
	 * @return the url resolved within the first argument
	 * @author Artur Linhart
	 */
	static String resolveUrlStringFromIndex(String aStringContainingUrl,
			int aIndexOfFirstCheckedCharacter, String aMasterEndCharacter,
			boolean aIgnoreBeginCharPrefix) {
		int locNumberOfLeadingSpaces = 0;
		int locStringCheckPosition = aIndexOfFirstCheckedCharacter;
		while (locStringCheckPosition > -1) {
			if (locStringCheckPosition < aStringContainingUrl.length()
					&& " "
							.equals(aStringContainingUrl.substring(
									locStringCheckPosition,
									locStringCheckPosition + 1))) {
				locNumberOfLeadingSpaces = locNumberOfLeadingSpaces + 1;
				locStringCheckPosition = locStringCheckPosition + 1;
			} else
				locStringCheckPosition = -1;
		}
		aIndexOfFirstCheckedCharacter = aIndexOfFirstCheckedCharacter
				+ locNumberOfLeadingSpaces;
		String locBeginStringCharacter = "";
		String locEndStringCharacter = "";
		for (int locI = 0; locI < BEGIN_CHARS.length; locI++) {
			if (aStringContainingUrl.indexOf(BEGIN_CHARS[locI],
					aIndexOfFirstCheckedCharacter) == aIndexOfFirstCheckedCharacter) {
				locBeginStringCharacter = BEGIN_CHARS[locI];
				locEndStringCharacter = END_CHARS[locI];
			}

		}
		int locIndexOfFirstNonBorderCharacter = aIndexOfFirstCheckedCharacter
				+ locBeginStringCharacter.length();
		if ("".equals(locBeginStringCharacter)) {
			/*
			 * There has been found no begin character, so resolve the url,
			 * which has to end by the sign aMasterEndCharacter, but before
			 * this, check, if the string does not start by the prefix:
			 */
			int locArrayIndexOfAlternateBeginCharacter = -1;
			if (!aIgnoreBeginCharPrefix)
				for (int locI = 0; locI < ALTERNATE_BEGIN_CHAR_PREFIX.length; locI++) {
					String locRelevantFirstCharacters = "";
					if (locIndexOfFirstNonBorderCharacter
							+ ALTERNATE_BEGIN_CHAR_PREFIX[locI].length() < aStringContainingUrl
							.length())
						locRelevantFirstCharacters = aStringContainingUrl
								.substring(
										locIndexOfFirstNonBorderCharacter,
										locIndexOfFirstNonBorderCharacter
												+ ALTERNATE_BEGIN_CHAR_PREFIX[locI]
														.length());
					if (locRelevantFirstCharacters.toUpperCase().equals(
							ALTERNATE_BEGIN_CHAR_PREFIX[locI].toUpperCase()))
						/*
						 * If the substring "starts with" - comparing in the
						 * case insensitive way - the alternate begin char
						 * prefix, then remember the prefix index:
						 */
						locArrayIndexOfAlternateBeginCharacter = locI;
				}
			if (locArrayIndexOfAlternateBeginCharacter >= 0) {
				locIndexOfFirstNonBorderCharacter = locIndexOfFirstNonBorderCharacter
						+ ALTERNATE_BEGIN_CHAR_PREFIX[locArrayIndexOfAlternateBeginCharacter]
								.length();
				return resolveUrlStringFromIndex(aStringContainingUrl,
						locIndexOfFirstNonBorderCharacter, "", true);
			} else {
				int locIndexOfEndCharacter = aStringContainingUrl.indexOf(
						aMasterEndCharacter, locIndexOfFirstNonBorderCharacter);
				if (locIndexOfEndCharacter < 0)
					return "";
				else
					return aStringContainingUrl.substring(
							locIndexOfFirstNonBorderCharacter,
							locIndexOfEndCharacter).trim();
			}
		} else {
			/*
			 * There has been found the start character - so process the method
			 * again - - this solves not only the url starting with '(' and '"'
			 * but also by '("' or '( "' or similar etc. - ignore the prefix
			 * before the border characters:
			 */
			return resolveUrlStringFromIndex(aStringContainingUrl,
					locIndexOfFirstNonBorderCharacter, locEndStringCharacter,
					true);

		}
	}

	/**
	 * Function resolves the url which can come from the begin. The String can
	 * begin with some empty spaces or brackett or doublequotes The result
	 * scring must then end by the corrresponding "closing" sign
	 * 
	 * @param aStringContainingUrlFromBegin
	 * @return the url string which first characters and ended before the end
	 *         character.
	 */
	static String resolveUrlStringFromBegin(String aStringContainingUrlFromBegin) {
		if (aStringContainingUrlFromBegin == null)
			return null;
		/* first resolve how many leading spaces are there from the beginning: */
		return resolveUrlStringFromIndex(aStringContainingUrlFromBegin, 0, "",
				false);
	}
	
	private static FileImpl _fileUtil = new FileImpl();
	
	// private static final String IMPORT_BEGIN = "@import url(";
	private static final String IMPORT_BEGIN = "@import";
	private static final String IMPORT_END = ");";
	private static final String URL_BEGIN_PREFIX = "url";
	private static final int URL_BEGIN_PREFIX_LENGTH = URL_BEGIN_PREFIX
			.length();
	private static final String URL_BEGIN_CHAR = "(";

	private static final String[] BEGIN_CHARS = new String[] { "(", "\"" };
	private static final String[] END_CHARS = new String[] { ")", "\"" };
	private static final String[] ALTERNATE_BEGIN_CHAR_PREFIX = new String[] {
			"url", "URL" };

}
